;NSIS Modern User Interface
;Christmas Countdown Installation File
;Written by Keith Survell

;--------------------------------
;Compression method

  SetCompressor /FINAL /SOLID lzma

;--------------------------------
;Include Modern UI

  !include "MUI.nsh"
  ;!include "Library.nsh"
  !include WordFunc.nsh
  !insertmacro VersionCompare

  !include LogicLib.nsh

  !define VERSION "1.04"
  
  ; Sets the theme path
  !define OMUI_THEME_PATH "${NSISDIR}\Contrib\MUIOrangeVistaTheme\CD-Clean"

  ; MUI Settings / Icons

  ; In the moment of writing this, NSIS don't support well Vista icons with PNG compression.
  ; We provide both, compressed and uncompressed (-nopng) icons.

  !define MUI_ICON "${OMUI_THEME_PATH}\installer-nopng.ico"
  !define MUI_UNICON "${OMUI_THEME_PATH}\uninstaller-nopng.ico"

  ; MUI Settings / Header
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_RIGHT
  !define MUI_HEADERIMAGE_BITMAP "${OMUI_THEME_PATH}\header-r.bmp"
  !define MUI_HEADERIMAGE_UNBITMAP "${OMUI_THEME_PATH}\header-r-un.bmp"

  ; MUI Settings / Wizard
  !define MUI_WELCOMEFINISHPAGE_BITMAP "${OMUI_THEME_PATH}\wizard.bmp"
  !define MUI_UNWELCOMEFINISHPAGE_BITMAP "${OMUI_THEME_PATH}\wizard-un.bmp"

;--------------------------------
;General

  ;Name and file
  Name "Christmas Countdown"
  OutFile "C:\Users\Keith\Documents\starkeith.net\coredump\files\cc\setup.exe"

  XPStyle on
  BrandingText "Christmas Countdown ${VERSION} Setup"

  ;Default installation folder
  InstallDir "$PROGRAMFILES\ChristmasCountdown"
  
  Var "DOTNET_RETURN_CODE"
  
;--------------------------------
;Functions

  Function .onInit
           InitPluginsDir
   FunctionEnd
   
   Function GetDotNETVersion
            Push $0
            Push $1
            System::Call "mscoree::GetCORVersion(w .r0, i ${NSIS_MAX_STRLEN}, *i) i .r1 ?u"
            StrCmp $1 0 +2
            StrCpy $0 "not found"

            Pop $1
            Exch $0
   FunctionEnd

;--------------------------------
;Interface Settings

  !define MUI_ABORTWARNING

;--------------------------------
;Pages

  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE "license.rtf"
  !insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  
;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections

Section "Pre-Install Section" PreSetup

    Call GetDotNETVersion
    Pop $0
    ${If} $0 == "not found"
          goto dodl
    ${EndIf}

    StrCpy $0 $0 "" 1 # skip "v"

    ${VersionCompare} $0 "2.0" $1
    ${If} $1 == 2
        goto dodl
    ${EndIf}
    ; if we got here, then the version is OK
    goto nodl
    
dodl:
     MessageBox MB_YESNO|MB_ICONQUESTION "This program requires the .NET Framework 2.0. Do you want to download and install \
     The .NET Framework now?" /SD IDYES IDNO dlcancel
     ; DO the download
     InetLoad::load /RESUME "The download has been inturrupted - you may have lost your connection \
     to the Internet. Please make sure you are connected and click Retry to retry the download." "http://download.microsoft.com/download/a/a/c/aac39226-8825-44ce-90e3-bf8203e74006/dotnetfx.exe" "$TEMP\dotnetfx.exe"
     ; maybe use $TEMP instead of $EXEDIR?
     Pop $0 # return value = exit code, "OK" if OK, otherwise display an error.
     StrCmp $0 "OK" dlok
     MessageBox MB_OK|MB_ICONEXCLAMATION "Download Error, click OK to abort installation" /SD IDOK
     Abort "Download Error"
     
dlcancel:
     MessageBox MB_OK|MB_ICONEXCLAMATION "You have chosen not to install the .NET Framework. The installation will now exit. \
     If you want to install the .NET Framework yourself, please do so before running setup again." /SD IDOK
     Abort ".NET Framework Installation Canceled"
    
dlok:
     ; downloaded OK, so run
     StrCpy $DOTNET_RETURN_CODE "0"
     DetailPrint "Installing the .NET Framework 2.0 - this may take a few minutes - please wait..."
     nsExec::ExecToStack '"$TEMP\dotnetfx.exe" /q /c:"install.exe /q"'
     pop $DOTNET_RETURN_CODE
     
     StrCmp "$DOTNET_RETURN_CODE" "" lbl_NoError
     StrCmp "$DOTNET_RETURN_CODE" "0" lbl_NoError
     StrCmp "$DOTNET_RETURN_CODE" "3010" lbl_RebootRequired
     StrCmp "$DOTNET_RETURN_CODE" "8192" lbl_RebootRequired
     StrCmp "$DOTNET_RETURN_CODE" "error" lbl_Error
     StrCmp "$DOTNET_RETURN_CODE" "timeout" lbl_TimeOut
     ; It's a .Net Error
     StrCmp "$DOTNET_RETURN_CODE" "4101" lbl_Error_DuplicateInstance
     StrCmp "$DOTNET_RETURN_CODE" "4097" lbl_Error_NotAdministrator
     StrCmp "$DOTNET_RETURN_CODE" "1633" lbl_Error_InvalidPlatform lbl_FatalError
     ; all others are fatal

    lbl_Error_DuplicateInstance:
    DetailPrint "The .NET Framework Installer is already running."
    GoTo lbl_Done

    lbl_Error_NotAdministrator:
    DetailPrint "You do not have the necessary privilages to install the .NET Framework.\
    On Windows NT or greater (2000, XP, 2003, and Vista) you must have administrative\
    privilages to install the .NET Framework."
    GoTo lbl_Done

    lbl_Error_InvalidPlatform:
    DetailPrint "This version of Windows is not supported by the .NET Framework."
    GoTo lbl_Done

    lbl_TimeOut:
    DetailPrint "The installation of the .NET Framework has timed out."
    GoTo lbl_Done

    lbl_Error:
    DetailPrint "The .NET Framework installer (dotnetfx.exe) was not found."
    GoTo lbl_Done

    lbl_FatalError:
    DetailPrint "A fatal error occurred during the installation of the .NET Framework: $DOTNET_RETURN_CODE"
    GoTo lbl_Done
    
    lbl_RebootRequired:
    DetailPrint "Installation of the .NET Framework is complete; but a reboot will be required."
    SetRebootFlag True
    GoTo nodl

    lbl_Done:
    ; come here after all fatal errors of the .net framework
    MessageBox MB_OK|MB_ICONSTOP "The installation of the .NET Framework has failed. This program requires the .NET Framework. \
    Please ensure that your computer meets the system requirements of this program and the .NET Framework 2.0 before trying setup again."
    Abort ".NET Framework Setup Failed"

lbl_NoError:
    ; no error
    ; remove the file we downloaded
    Delete "$TEMP\dotnetfx.exe"

nodl:
    ; no download needed

SectionEnd

Section "Main Section" SecMain

  ; Gotta love these XCOPY deployments
  ; just copy everything from the release folder into the installation folder
  SetOutPath "$INSTDIR"
  SetOverwrite Try
  File /a /r "C:\Users\Keith\Documents\Visual Studio 2005\Projects\ChristmasCountdown\bin\Release\*.*"
  IfErrors 0 NoOverwriteErrors
     ; if there was an overwrite error, extract to a temp folder
     ; and use the rename /rebootok function to extract the files
     ; and force a reboot (if needed)
     ClearErrors
     SetOutPath "$TEMP"
     SetOverwrite on ; we always want to overwrite files in the temp folder
     File /a /oname=ChristmasCountdown.exe "C:\Users\Keith\Documents\Visual Studio 2005\Projects\ChristmasCountdown\bin\Release\ChristmasCountdown.exe"
     ; rename the files (this is like "moving" the files)
     Rename /REBOOTOK "$TEMP\ChristmasCountdown.exe" "$INSTDIR\ChristmasCountdown.exe"
NoOverwriteErrors:
  ;Create uninstaller
  WriteUninstaller "$INSTDIR\Uninstall.exe"
  
  ; Start Menu Items
  SetShellVarContext all
  CreateDirectory "$SMPROGRAMS\ChristmasCountdown"
  CreateShortCut "$SMPROGRAMS\ChristmasCountdown\Christmas Countdown.lnk" "$INSTDIR\ChristmasCountdown.exe" "" ""
  CreateShortCut "$SMPROGRAMS\ChristmasCountdown\Remove Christmas Countdown.lnk" "$INSTDIR\Uninstall.exe" "" ""
  
  ;Create Add/Remove Programs Entry
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "DisplayName" "Christmas Countdown Timer"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "UninstallString" "$INSTDIR\uninstall.exe"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "Publisher" "Keith Enterprises"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "HelpLink" "http://www.starkeith.net/coredump/"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "URLInfoAbout" "http://www.starkeith.net"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "DisplayIcon" "$INSTDIR\ChristmasCountdown.exe"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "DisplayVersion" "${VERSION}"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown" "NoRepair" 1
  
  IfRebootFlag 0 norebootrequired
  MessageBox mb_yesno|mb_iconquestion "Your computer must be restarted in order to complete the installation. Do you want to reboot now?" idno noreboot
  Reboot
noreboot:
  MessageBox MB_OK|MB_ICONINFORMATION "The installation will be complete the next time you reboot your computer."
norebootrequired:
SectionEnd

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  ; Remove the installation folder
  Delete /REBOOTOK "$INSTDIR\*.*"
  ; remove shortcuts
  SetShellVarContext all
  RMDir /r "$SMPROGRAMS\ChristmasCountdown"
  ; remove the Registry entries (for Add/Remove Programs)
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ChristmasCountdown"
SectionEnd