Imports System.Net, System.IO

Public Class frmMain

    Dim realClose As Boolean = False

    Private Sub myNotifyIcon_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles myNotifyIcon.MouseDoubleClick
        Dim daysTillChristmas As String = calculateDaysTillChristmas()
        myNotifyIcon.BalloonTipIcon = ToolTipIcon.Info
        myNotifyIcon.BalloonTipTitle = "Christmas Countdown"
        myNotifyIcon.BalloonTipText = "There are " & daysTillChristmas & " days until Christmas."
        myNotifyIcon.ShowBalloonTip(10000)
    End Sub

    Private Function calculateDaysTillChristmas() As String
        Dim dayDiff As Integer
        If Date.Today.Month = 12 And Date.Today.Day > 25 Then
            ' we've passed christmas, but haven't passed into the new year yet
            dayDiff = DateDiff(DateInterval.Day, Date.Today, New Date(Date.Today.Year + 1, 12, 25))
        Else
            ' calculate days from NOW until Dec 25th of NOW.YEAR
            dayDiff = DateDiff(DateInterval.Day, Date.Today, New Date(Date.Today.Year, 12, 25))
        End If
        Return dayDiff.ToString
    End Function

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        realClose = True
        Me.Close()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        Dim myWindow As New frmAbout
        frmAbout.ShowDialog()
        myWindow = Nothing
    End Sub

    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' check WHY we are closing
        If realClose = False Then
            ' if realClose was true, we'd really close
            If e.CloseReason = CloseReason.UserClosing Then
                ' we've clicked the "close" button
                ' don't close, just hide
                e.Cancel = True
                Me.Visible = False
            End If
        End If
    End Sub

    Private Sub frmMain_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        Me.Visible = False
        myWorker.RunWorkerAsync()
        If Date.Today <> My.Settings.LastTipDate Then
            ' notify people of our tray icon
            myNotifyIcon.BalloonTipIcon = ToolTipIcon.Info
            myNotifyIcon.BalloonTipTitle = "Christmas Countdown"
            myNotifyIcon.BalloonTipText = "Double-click this icon to see the countdown." & vbNewLine & "Right-click to see the menu and to exit."
            myNotifyIcon.ShowBalloonTip(5000)
            ' and remember when we last showed it 
            My.Settings.LastTipDate = Date.Today
        End If
        Dim daysTillChristmas As String = calculateDaysTillChristmas()
        myNotifyIcon.Text = "There are " & calculateDaysTillChristmas() & " days until Christmas."
    End Sub

    Private Sub ShowCountdownToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShowCountdownToolStripMenuItem.Click
        ' show the countdown screen
        Dim daysTillChristmas As String = calculateDaysTillChristmas()
        lblCountdown.Text = "There are " & calculateDaysTillChristmas() & " days until Christmas."
        If Date.IsLeapYear(Date.Today.Year) = True Then
            myProgressBar.Maximum = 366
        Else
            myProgressBar.Maximum = 365
        End If

        myProgressBar.Value = myProgressBar.Maximum - CInt(daysTillChristmas)
        myProgressBar.Style = ProgressBarStyle.Continuous
        Me.Visible = True
    End Sub

    Private Sub myWorker_DoWork(ByVal sender As Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles myWorker.DoWork
        ' check for update
        Dim myRequest As HttpWebRequest = WebRequest.Create("http://www.starkeith.net/coredump/files/cc/version.txt")

        Try
            ' get the response
            Dim myResponse As HttpWebResponse = myRequest.GetResponse()

            ' convert to string and check it
            Dim myStringReader As New StreamReader(myResponse.GetResponseStream)
            Dim theResults As String = myStringReader.ReadToEnd()

            ' now check it
            Dim webVersion As New Version(theResults)
            If My.Application.Info.Version < webVersion Then
                ' web version is newer
                ' download it

                ' figure out where we're going to save it
                Dim myUpdateFileName As String = Path.Combine(My.Computer.FileSystem.SpecialDirectories.CurrentUserApplicationData, "update.exe")
                ' and where it is on the web
                myRequest = WebRequest.Create("http://www.starkeith.net/coredump/files/cc/setup.exe")

                ' now try it
                Try
                    myResponse = myRequest.GetResponse()

                    Dim mySize As Long = myResponse.ContentLength()

                    ' get ready to save the file
                    Dim myReader As New BinaryReader(myResponse.GetResponseStream())
                    ' see if it's there already
                    If File.Exists(myUpdateFileName) = True Then
                        ' is it exactly the same size?
                        Dim tmp As New FileInfo(myUpdateFileName)
                        If tmp.Length = mySize Then
                            ' the two files are exactly the same size
                            ' probably the same update
                            ' so quit
                            myReader.Close()
                            Exit Sub
                        Else
                            ' not the same size; so delete it
                            File.Delete(myUpdateFileName)
                        End If
                    End If


                    ' create the file on disk
                    Dim myFile As New FileStream(myUpdateFileName, FileMode.CreateNew)
                    If My.Computer.FileSystem.DirectoryExists(My.Computer.FileSystem.SpecialDirectories.CurrentUserApplicationData) = False Then
                        My.Computer.FileSystem.CreateDirectory(My.Computer.FileSystem.SpecialDirectories.CurrentUserApplicationData)
                    End If
                    Dim myFileWriter As New BinaryWriter(myFile)

                    ' write the whole thing out to the file
                    myFileWriter.Write(myReader.ReadBytes(mySize))

                    ' close the file
                    myFile.Close()
                    ' and the stream
                    myReader.Close()

                    ' return the name of the update file on disk so we can run it
                    e.Result = myUpdateFileName
                    Exit Sub
                Catch ex As Exception
                    ' an exception downloading the file
                    My.Application.Log.WriteException(ex)
                    e.Result = False
                End Try
                
            End If
        Catch ex As Exception
            ' an exception getting the version information
            My.Application.Log.WriteException(ex)
            e.Result = False
        End Try
    End Sub

    Private Sub myWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles myWorker.RunWorkerCompleted
        ' we've finished doing the update
        CheckForUpdatesToolStripMenuItem.Visible = True
        If TypeOf e.Result Is String Then
            ' try to run it
            If File.Exists(e.Result.ToString()) = True Then
                ' save the file name
                My.Settings.UpdateFileName = e.Result.ToString()
                ' activate the menu item
                InstallUpdateToolStripMenuItem.Visible = True
                ' and display a notice
                myNotifyIcon.BalloonTipIcon = ToolTipIcon.Info
                myNotifyIcon.BalloonTipTitle = "Update Available"
                myNotifyIcon.BalloonTipText = "A new update is available. Right-click and choose" & vbNewLine & _
                                              "'Install Update' to install the update."
                myNotifyIcon.ShowBalloonTip(30000)
            End If
        Else
            ' no update detected, but check to see if we downloaded one previously 
            If My.Settings.UpdateFileName.Length > 0 Then
                ' yep, we've got something
                ' probably from the last time we were loaded
                ' so make the menu item visible
                InstallUpdateToolStripMenuItem.Visible = True
                ' and display a notice
                myNotifyIcon.BalloonTipIcon = ToolTipIcon.Info
                myNotifyIcon.BalloonTipTitle = "Update Available"
                myNotifyIcon.BalloonTipText = "A new update is available. Right-click and choose" & vbNewLine & _
                                              "'Install Update' to install the update."
                myNotifyIcon.ShowBalloonTip(30000)
            End If
        End If
    End Sub

    Private Sub InstallUpdateToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InstallUpdateToolStripMenuItem.Click
        Try
            System.Diagnostics.Process.Start(My.Settings.UpdateFileName)
        Catch ex As Exception
            My.Application.Log.WriteException(ex)
        Finally
            ' hide the menu item
            InstallUpdateToolStripMenuItem.Visible = False
            ' and clear this setting
            My.Settings.UpdateFileName = String.Empty
        End Try
    End Sub

    Private Sub myTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles myTimer.Tick
        ' update the tooltip text for the tray icon
        Dim daysTillChristmas As String = calculateDaysTillChristmas()
        myNotifyIcon.Text = "There are " & calculateDaysTillChristmas() & " days until Christmas."
    End Sub

    Private Sub CheckForUpdatesToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckForUpdatesToolStripMenuItem.Click
        If myWorker.IsBusy = False Then
            ' only run it if it's not currently running, of course
            myWorker.RunWorkerAsync()
            myNotifyIcon.BalloonTipIcon = ToolTipIcon.Info
            myNotifyIcon.BalloonTipTitle = "Checking for updates..."
            myNotifyIcon.BalloonTipText = "Checking for updates. You will be notified if a new update is available."
            myNotifyIcon.ShowBalloonTip(10000)
            ' and hide our menu item until it's done
            CheckForUpdatesToolStripMenuItem.Visible = False
        End If
    End Sub
End Class
